package grizzly.tutorial.conncaching.client;


import static grizzly.tutorial.conncaching.Utils.*;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;

/**
 * This class maintains a socket per server end point and is used to send
 * out-bound messages (as requests). We use a temporary write selector for this
 * prupose. As a client-side channel, thread-safe is the caller's responsibility.
 * 
 * @author wenboz
 */
public class RequestChannel {

	private SocketChannel sc;

	private InetSocketAddress serverId;

	private ByteBuffer buffer = ByteBuffer.allocateDirect(1024);

	public RequestChannel(InetSocketAddress serverId) {
		this.serverId = serverId;
	}

	public void open() throws Exception {
		sc = SocketChannel.open();
		sc.connect(serverId);

		sc.configureBlocking(false);
	}

	public void sendRequest(String serverObjectId, Object msg, InetSocketAddress clientId) throws Exception {
		
		send(clientId.toString() + "\n" + serverObjectId + "\n" + msg.toString() + PAYLOAD_DELIMITER); 
	}

	private void send(String msg) throws Exception {
		buffer.clear();
		buffer.put(msg.getBytes());
		buffer.flip();

		Selector writeSelector = Selector.open();

		try {
			while (buffer.hasRemaining()) {
				int len = sc.write(buffer);
				if (len == 0) {
					sc.register(writeSelector, SelectionKey.OP_WRITE);
					if (writeSelector.select(30 * 1000) == 0) {
						throw new IOException("Client disconnected");
					}
				}
			}
		} finally {
			writeSelector.close();
		}

	}

	public void close() {
		if (sc != null) {
			try {
				sc.close();
			} catch (Exception ex) {
				trace(ex);
			} finally {
				sc = null;
			}
		}
	}

	public InetSocketAddress getServerId() {
		return serverId;
	}
}
