package grizzly.tutorial.conncaching.client;

import static grizzly.tutorial.conncaching.Utils.*;

import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * client is sequentially invoking server objects, asynchronously.
 * it uses a thread-managed conn. cache to enable demonstrating  
 * outbound conn. caching on the server side.
 * 
 * @author wenboz
 */
public class SimpleMORBClientApp {

	private ResponseDispatchChannel in;
	
	private Map<InetSocketAddress, RequestChannel> requestChannelCache = new HashMap<InetSocketAddress, RequestChannel>();

	private List<InetSocketAddress> servers = new ArrayList<InetSocketAddress>();

	private Thread dispatchThread;

	public static void main(String[] args) throws Exception {

		if (args.length < 2) {
			trace("Usage: SimpleClientApp <local ip>:<local port> <server ip>:<server port> [<server ip>:<server port>]");
			return;
		}

		SimpleMORBClientApp app = new SimpleMORBClientApp();

		app.init(args[0]);

		for (int i = 1; i < args.length; i++) {
			app.register(resolveAddressParam(args[i]));
		}

		app.start();
		
		app.doSomething();

		app.end();
	}

	public void init(String localAddr) throws Exception {
		in = new ResponseDispatchChannel(resolveAddressParam(localAddr));
		in.open();
	}

	public void register(final InetSocketAddress serverId) throws Exception {
		servers.add(serverId);
		in.register(serverId, new ResponseHandler() {

			public void handle(String response) throws Exception {
				trace("response from " + serverId + ":" + response);
			}
		});
	}

	public void start() {

		dispatchThread = new Thread(new Runnable() {

			public void run() {

				try {
					in.dispatch();
				} catch (InterruptedException ex) {
					trace("dispatch thread interrupted");
				} catch (Exception ex) {
					trace(ex);
				}
			}
		});

		dispatchThread.start();
	}

	public void end() {

		dispatchThread.interrupt();
		try {
			dispatchThread.join();
		} catch (InterruptedException ex) {
		}
		
		for (RequestChannel channel : requestChannelCache.values()) {
			channel.close();
		}

		in.close();
	}

	public void doSomething() {

		RequestChannel out;
		int i;
		i = 0;
		String msg;
		do {
			for (InetSocketAddress server : servers) {
				
				msg = in.getClientId().toString() + "[" + i++ + "]" + "as;ldfjas;ldjkfasljfaslk;jfasldjflkasjflksajflksjadflkjsalkfjlskajdflksa;jflkjsldkfjsalkjfklasjfklsajfklsjafdlksajflksajfklsjdfkljsfkljsalkfjslkajfkl;sajfl;sajfl;sjfkls;ajflsjflsjfklsdjfklsajfkljsdklf;jal;sjdflksjflska;jkljsflk;asjflksajflksjflksajflksajdflksajflksajflksajflk;asjflk;sajfklsajf;laskjf;lasjflkasjfklsajfklasjfkl;asjflk;safjlksajflksajflksjflksajflksajfklsajflksajflksjaflksafjklsajfklsajfklsafkjslkajfkslajfslkajfklsaflksajflksjflksajfklsajfklsjfklsjflksafslakfjjaskl;fdlksajfklsjfklasjfklsafjskalfjlskafjlksa;jfla;ksjflksafjlskajf";
				out = requestChannelCache.get(server);
				try {
					if (out == null) {
						out = new RequestChannel(server);
						out.open();
					}
					out.sendRequest("server", msg, in.getClientId());
				} catch (Exception ex) {
					trace(ex);
					trace("retry: " + server);
					out.close();
					out = new RequestChannel(server);
					
					try {
						out.open();
						out.sendRequest("server", msg, in.getClientId());
					}
					catch (Exception exx) {
						trace(exx);
						trace("failed: " + server);
						out.close();
						continue;
					}
				}
				requestChannelCache.put(server, out);
			}
		} while (step("type c to continue") == 'c');
	}
}
