package grizzly.tutorial.conncaching.server;

import static grizzly.tutorial.conncaching.Utils.trace;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class instantiates a Messaging ORB, with only one single "echo" 
 * server object instance registered.
 *  
 * @author wenboz
 *
 */
public class MessagingORB {

	private ServerRequestController serverController;

	public static void main(String[] args) throws Exception {

		if (args.length < 1) {
			trace("Usage: MessageORB <local ip>:<local port>");
			return;
		}


		Logger logger = Logger.getLogger("grizzly");
		logger.setLevel(Level.ALL);

		MessagingORB morb = new MessagingORB();

		morb.init(args[0]);

		morb.deploy();  

		morb.open(args[0]);

		morb.close();  
	}

	public void init(String localAddr) throws Exception {
		
		trace("init ...");
		serverController = new ServerRequestController("server-" + localAddr);
	}

	public void open(String addr) throws Exception {
		
		trace("open ..." + addr);
		serverController.open(addr);
	}

	public void close() {
		trace("close ...");
		serverController.close();
	}

	public void deploy() {
		
		trace("deploy ...");
		serverController.register("server", new RequestHandler() {
			public String handle(String request) throws Exception {
				trace("invoke " + request);
				return "request handled: " + request;
			}
		});

	}
	
}
